//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <io.h>
#include <stdio.h>
#include "zcrypt.h"
//#include "crc32.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

static AnsiString PQANm = "pqa.gfd";
static AnsiString RandSBXNm = "prnd.sbx";
static AnsiString SignSBXNm = "sign.sbx";
static AnsiString GOSTSBXNm = "gost.sbx";

__fastcall TZvitCrypt::TZvitCrypt()
{
   HEncrypt = NULL;
   LoadPQA = NULL;
   LoadCryptSBox = LoadSignSBox = LoadRandSBox = NULL;
   ZvitEncryptMsg = ZvitDecryptMsg = NULL;
   InitializeRndGenerator = NULL;
   memset(&SignKey,0,sizeof(SignKey));
   memset(&SignKey2,0,sizeof(SignKey2));
   memset(&CryptKey,0,sizeof(CryptKey));
   SignKeyLen = SignKey2Len = CryptKeyLen = 0;
   OnGetQwrHeader = NULL;
}
__fastcall TZvitCrypt::~TZvitCrypt()
{
   if(HEncrypt) FreeLibrary(HEncrypt);
}

bool __fastcall TZvitCrypt::LoadSignKey(AnsiString FileName)
{
  TFileStream * key = NULL;
  memset(&SignKey,0,sizeof(SignKey));
  try
  {
        key = new TFileStream(FileName, fmOpenRead);
  }
  catch (...)
  {
        return false;
  }
  SignKeyLen = key->Size;
  key->Read(&SignKey,SignKeyLen);
  delete key;
  return true;
}

bool __fastcall TZvitCrypt::LoadSign2Key(AnsiString FileName)
{
  TFileStream * key = NULL;
  //LogMessage(AnsiString("LoadSign2Key from "+FileName).c_str());
  memset(&SignKey2,0,sizeof(SignKey2));
  SignKey2Len = 0;
  if(!FileExists(FileName)) return false;
  try
  {
        //LogMessage("LoadSign2Key before load");
        key = new TFileStream(FileName, fmOpenRead);
  }
  catch (...)
  {
        return false;
  }
  SignKey2Len = key->Size;
  //LogMessage("LoadSign2Key before read");
  key->Read(&SignKey2,SignKey2Len);
  delete key;
  //LogMessage("LoadSign2Key end");
  return true;
}

bool __fastcall TZvitCrypt::LoadCryptKey(AnsiString FileName)
{
  TFileStream * key = NULL;
  memset(&CryptKey,0,sizeof(CryptKey));
  try
  {
        key = new TFileStream(FileName, fmOpenRead);
  }
  catch (...)
  {
        return false;
  }
  CryptKeyLen = key->Size;
  key->Read(&CryptKey,CryptKeyLen);
  delete key;
  return true;
}
long GetFileSize(const char *fname)
{
   long fsize;
   int handle;
   handle = open(fname, 0);
   fsize = filelength(handle);
   close(handle);
   return fsize;
}

bool LoadFile(char* filename, char * fbuf, int& FileSize)
{
   FILE *pfile;
   FileSize = GetFileSize(filename);
   if(!FileSize) return false;
   pfile = fopen(filename,"rb");
   if(!pfile) return NULL;
   fread(fbuf,FileSize,1,pfile);
   fclose(pfile);
   return true;
}

bool __fastcall TZvitCrypt::LoadHashFiles(AnsiString Dir)
{
  if(!InitCryptLib()) return false;
  // PQA
  AnsiString PQAFileName = Dir+PQANm;
  LoadPQA(PQAFileName.c_str());
  LoadFile(PQAFileName.c_str(),PQAGFD,PQAGFDLen);
  // RandSBox
  AnsiString RandSBXFileName = Dir+RandSBXNm;
  LoadRandSBox(RandSBXFileName.c_str());
  LoadFile(RandSBXFileName.c_str(),RandSBX,RandSBXLen);
  // SignSBox
  AnsiString SignSBXFileName = Dir+SignSBXNm;
  LoadSignSBox(SignSBXFileName.c_str());
  LoadFile(SignSBXFileName.c_str(),SignSBX,SignSBXLen);
  // CryptSBox
  AnsiString GOSTSBXFileName = Dir+GOSTSBXNm;
  LoadCryptSBox(GOSTSBXFileName.c_str());
  LoadFile(GOSTSBXFileName.c_str(),CryptSBX,CryptSBXLen);
  if(!InitializeRndGenerator()) return false;
  return true;
}
bool __fastcall TZvitCrypt::AskCryptKeys(AnsiString Dir)
{
  memset(&SignKey,0,sizeof(SignKey));
  memset(&SignKey2,0,sizeof(SignKey2));
  memset(&CryptKey,0,sizeof(CryptKey));
  SignKeyLen = SignKey2Len = CryptKeyLen = 0;

  //  
  AnsiString AccSignKey = Dir + "\\private_accountant" + "\\bsignkey.gsk";
  if (!LoadSign2Key(AccSignKey))
  {
    MessageBox(NULL,
               ("   " + AccSignKey).c_str(),
               "", MB_OK|MB_ICONEXCLAMATION);
    return false;
  }

  //  
  AnsiString ManSignKey = Dir + "\\private_manager" + "\\dsignkey.gsk";
  AnsiString ManCryptKey = Dir + "\\private_manager" + "\\ncrpkey.gpk";
  AnsiString HashDir = Dir + "\\private_manager\\";
  if (!LoadSignKey(ManSignKey) ||
      ! LoadCryptKey(ManCryptKey) ||
      ! LoadHashFiles(HashDir))
  {
    MessageBox(NULL,
               ("      -: " +
               ManSignKey + ", " +
               ManCryptKey + ", " +
               HashDir).c_str(),
               "", MB_OK|MB_ICONEXCLAMATION);
    return false;
  }
  
  return true;
}

bool __fastcall TZvitCrypt::AskDecryptRplKeys(AnsiString Dir)
{
  memset(&SignKey,0,sizeof(SignKey));
  memset(&SignKey2,0,sizeof(SignKey2));
  memset(&CryptKey,0,sizeof(CryptKey));
  SignKeyLen = SignKey2Len = CryptKeyLen = 0;

  //AnsiString EDRPOU = Pck->LoadCryptHeader()->Adr.NP.To;
  LoadSign2Key(""); // clear key
  
  //  
  AnsiString ManSignKey = Dir + "\\private_manager" + "\\nsignkey.gpk";
  AnsiString ManCryptKey = Dir + "\\private_manager" + "\\dcrpkey.gsk";
  AnsiString HashDir = Dir + "\\private_manager\\";
  if (!LoadSignKey(ManSignKey) ||
      ! LoadCryptKey(ManCryptKey) ||
      ! LoadHashFiles(HashDir))
  {
    MessageBox(NULL,
               ("      -: " +
               ManSignKey + ", " +
               ManCryptKey + ", " +
               HashDir).c_str(),
               "", MB_OK|MB_ICONEXCLAMATION);
    return false;
  }

  return true;
}

bool __fastcall TZvitCrypt::AskDecryptArcKeys(AnsiString Dir)
{
  LoadSign2Key(""); // clear key
  //  
  AnsiString ManSignKey = Dir + "\\private_manager" + "\\nsignkey.gpk";
  AnsiString ManCryptKey = Dir + "\\private_manager" + "\\dcrpkey.gsk";
  AnsiString HashDir = Dir + "\\private_manager\\";
  if (!LoadSignKey(ManSignKey) ||
      ! LoadCryptKey(ManCryptKey) ||
      ! LoadHashFiles(HashDir))
  {
    MessageBox(NULL,
               ("      -: " +
               ManSignKey + ", " +
               ManCryptKey + ", " +
               HashDir).c_str(),
               "", MB_OK|MB_ICONEXCLAMATION);
    return false;
  }

  return true;
}

bool __fastcall TZvitCrypt::AskDecryptPckKeys(AnsiString Dir)
{
  LoadSign2Key(""); // clear key
  //  
  AnsiString ManSignKey = Dir + "\\private_manager" + "\\nsignkey.gpk";
  AnsiString ManCryptKey = Dir + "\\private_manager" + "\\dcrpkey.gsk";
  AnsiString HashDir = Dir + "\\private_manager\\";
  if (!LoadSignKey(ManSignKey) ||
      ! LoadCryptKey(ManCryptKey) ||
      ! LoadHashFiles(HashDir))
  {
    MessageBox(NULL,
               ("      -: " +
               ManSignKey + ", " +
               ManCryptKey + ", " +
               HashDir).c_str(),
               "", MB_OK|MB_ICONEXCLAMATION);
    return false;
  }

  return true;
}
//---------------------------------------------------------------------
bool __fastcall TZvitCrypt::InitCryptLib(void)
{
  if(HEncrypt) return true;
  AnsiString appdir = ExtractFilePath(ParamStr(0))+"ZCrypt.dll";
  HEncrypt = ::LoadLibrary(appdir.c_str());
  if(!HEncrypt) {/*ShowLastError();*/return false;}
  LoadPQA = (PLoadPQA )::GetProcAddress(HEncrypt, "LoadPQA");
  LoadSignSBox = (PLoadSignSBox )::GetProcAddress(HEncrypt, "LoadSignSBox");
  LoadCryptSBox = (PLoadCryptSBox )::GetProcAddress(HEncrypt, "LoadCryptSBox");
  LoadRandSBox = (PLoadRandSBox )::GetProcAddress(HEncrypt, "LoadRandSBox");
  InitializeRndGenerator = (PInitializeRndGenerator )::GetProcAddress(HEncrypt, "InitializeRndGenerator");
//  GetKeyPair = (PGetKeyPair )::GetProcAddress(HEncrypt, "GetKeyPair");
  ZvitEncryptMsg = (PZvitEncryptMsg )::GetProcAddress(HEncrypt, "ZvitEncryptMsg");
  ZvitDecryptMsg = (PZvitDecryptMsg )::GetProcAddress(HEncrypt, "ZvitDecryptMsg");
  if(!LoadPQA || !ZvitEncryptMsg||!ZvitDecryptMsg)
  {  /*ShowLastError(); */return false;}

  return true;
}

uint4 s_Crc32(unsigned char *pchBuf, int nBufLen)
{
  unsigned long res = 0xFFFFFFFF;
  for(int i=0;i<nBufLen;i++)
  {
    res ^= (unsigned char)pchBuf[i];
    for(int j = 0; j < 8; ++j)
      res = (res>>1) ^ (0xEDB88320 * (res & 1));
  }

  return res;
}

bool __fastcall TZvitCrypt::Encrypt(AnsiString EDRPOU, TMemoryStream * File)
{
  if(!File) return false;
  if(SignKeyLen!=SECRETKEY_LEN || SignKey2Len!=SECRETKEY_LEN
   ||CryptKeyLen!=PUBLICKEY_LEN) //   
  {
        MessageBox(NULL,"   ","",MB_OK|MB_ICONERROR);
        return false;
  }
  if(!InitCryptLib()) return false;

  TRpgQwrHeader qwr;
  memset(&qwr,0,sizeof(qwr));
  strcpy(qwr.Sign,RPGQWRNEW_SIGN);
  qwr.Direct = RPGQWR_QWR_PLT2NAL;  //  
  strcpy(qwr.Adr.PN.From,EDRPOU.c_str());
  qwr.Adr.PN.To = 0;
  qwr.CryptKeyNum = 1;

//  AnsiString TaxNum = OrgDM->OrgTab->FindField("TaxInspNum")->AsString;
//  if(TaxNum.Length()>0) qwr.Adr.PN.To = TaxNum.ToInt();

  qwr.Type=2; //   hRpgEIPQTpPN2 -  ae  ii

  //   
  File->Seek(0,soFromBeginning);
  int size = File->Size;
  char * fbuf = new char[size+4096];
  memset(fbuf,0,size+4096);
  File->Read(fbuf,size);
  //    
  strcpy(fbuf+size,CRYPT_SIGN);
  size+=CRYPT_SIGN_SIZE;

  int MsgLen = size;  //     
  char * outbuf = ZvitEncryptMsg(SignKey,CryptKey, SignKey2, MsgLen, fbuf);
  delete fbuf;
  if(!outbuf)  return false;

  // CRC  -  
  qwr.Crc = ~0;
  qwr.Crc = s_Crc32((char*)&qwr, sizeof( TRpgQwrHeader ) );

  File->Seek(0,soFromBeginning);
  File->Write(&qwr,sizeof(TRpgQwrHeader));
  File->Write(&MsgLen,sizeof(int));
  File->Write(outbuf,MsgLen+16); // +16 -  

  return true;
}
bool __fastcall TZvitCrypt::Decrypt(TMemoryStream * File)
{
  if(!File) return false;

  //   
  //   TRpgQwrHeader
  static TRpgQwrHeader pQwrHeader;
  memset( &pQwrHeader, 0, sizeof(pQwrHeader) );
  File->Seek( 0, soFromBeginning );
  File->Read( &pQwrHeader, sizeof(pQwrHeader) );
  if ( OnGetQwrHeader && !OnGetQwrHeader( &pQwrHeader ) ) return false;

  if(SignKeyLen!=PUBLICKEY_LEN //|| SignKey2Len!=PUBLICKEY_LEN
   ||CryptKeyLen!=SECRETKEY_LEN) //   
  {
        MessageBox(NULL,"   ","",MB_OK|MB_ICONERROR);
        return false;
  }
  if(!InitCryptLib()) return false;


  //  
  /*File->Seek(sizeof(TRpgQwrHeader),soFromBeginning);*/
  int size = File->Size;
  File->Read(&size,4);
  char * fbuf = new char[File->Size+100];
  File->Read(fbuf,size);
  if(!strcmp(fbuf+size-CRYPT_SIGN_SIZE,CRYPT_SIGN)) //  ?!
  {
    size-=CRYPT_SIGN_SIZE;
    *(fbuf+size) = 0;
    File->Seek(0,soFromBeginning);
    File->Write(fbuf,size);
    File->Size = size;
    File->Seek(0,soFromBeginning);
    delete fbuf;
    return true;
  }
  char * outbuf = ZvitDecryptMsg( SignKey, CryptKey, *SignKey2 ? SignKey2 : NULL, size, fbuf );
  delete fbuf;
  if(!outbuf) return false;
  if(strcmp(outbuf+size-CRYPT_SIGN_SIZE,CRYPT_SIGN))
  {
      ShowMessage("    ");
      delete outbuf;
      return false;
  }
  size-=CRYPT_SIGN_SIZE;
  *(outbuf+size) = 0;
  File->Seek(0,soFromBeginning);
  File->Write(outbuf,size);
  File->Size = size;
  File->Seek(0,soFromBeginning);
  return true;
}

//-------------------------------------------------------------------------------
//        -
//-------------------------------------------------------------------------------
bool __fastcall TZvitCrypt::EncryptPlt2Nal( TStream * pStream, TRpgQwrHeader * pQwrHeader )
{
  if ( !InitCryptLib() || !AskCryptKeys( "A:\\" ) ) return false;

  //   
  char * pBuf = new char[ pStream->Size + 4096 ];
  memset( pBuf, 0, sizeof(pBuf) );
  pStream->Seek( 0, soFromBeginning );
  pStream->Read( pBuf, pStream->Size );
  int pBufSize = pStream->Size;

  //    
  strcpy( pBuf + pStream->Size, CRYPT_SIGN );
  pBufSize += CRYPT_SIGN_SIZE;

  int pMsgLen = pBufSize;
  char * pOutBuf = ZvitEncryptMsg( SignKey, CryptKey, SignKey2, pMsgLen, pBuf );
  delete pBuf;

  if ( !pOutBuf ) return false;

  //    
  pQwrHeader->Crc = ~0;
  pQwrHeader->Crc = s_Crc32((char*)pQwrHeader, sizeof( TRpgQwrHeader ) );

  pStream->Seek( 0, soFromBeginning );
  pStream->Write( pQwrHeader, sizeof(TRpgQwrHeader) );
  pStream->Write( &pMsgLen, sizeof(int) );
  pStream->Write( pOutBuf, pMsgLen/*+16*/ ); // +16 -  */

  char pDummy[8];
  memset( &pDummy, 0, sizeof(pDummy) );
  pStream->Write( pDummy, sizeof(pDummy) );

  return true;
}

//-------------------------------------------------------------------------------
//        -
//-------------------------------------------------------------------------------
bool __fastcall TZvitCrypt::DecryptNalFromPlt( TStream * pStream, TRpgQwrHeader * pQwrHeader )
{
  if ( !InitCryptLib() ) return false;
  /*
  LoadHashFiles( "A:\\" );

  TpFIBQuery * pOrgQry = InspDM->GetOrg( 0, pQwrHeader->Adr.PN.From, 1 );
  if ( !pOrgQry ) return false;

  int pOrgCode = pOrgQry->FieldByName( "Code" )->AsInteger;
  InspDM->SelPltKeys->ParamByName( "OrgCode" )->AsInteger = pOrgCode;
  InspDM->SelPltKeys->ExecQuery();
  if ( InspDM->SelPltKeys->Bof && InspDM->SelPltKeys->Eof ) return false;

  TFIBXSQLVAR * pDIRSig = InspDM->SelPltKeys->FieldByName( "DIRSig" );
  TFIBXSQLVAR * pBUHSig = InspDM->SelPltKeys->FieldByName( "BUHSig" );
  AnsiString pBlob = pDIRSig->AsString;
  int pBlobSz = pBlob.Length();
  //memcpy( CryptKey, pDCrpKey->Value.VPointer, pDCrpKey->Size );
  memcpy( &SignKey, pBlob.c_str(), pBlobSz );
  SignKeyLen = pBlobSz;
  pBlob = pBUHSig->AsString;
  pBlobSz = pBlob.Length();
  //memcpy( CryptKey, pDCrpKey->Value.VPointer, pDCrpKey->Size );
  memcpy( &SignKey2, pBlob.c_str(), pBlobSz );
  SignKey2Len = pBlobSz;

  int pSize = pStream->Size;
  pStream->Read( &pSize, 4 );
  char * pBuf = new char[ pStream->Size + 100 ];
  pStream->Read( pBuf, pSize );
  char * pOutBuf = ZvitDecryptMsg( SignKey, CryptKey, SignKey2, pSize, pBuf );
  delete pBuf;

  if ( !pOutBuf ) return false;

  if ( strcmp( pOutBuf + pSize - CRYPT_SIGN_SIZE, CRYPT_SIGN ) )
  {
    ShowMessage( "    " );
    delete pOutBuf;
    return false;
  }

  *(pOutBuf + pSize - CRYPT_SIGN_SIZE) = 0;
  pStream->Seek( 0, soFromBeginning );
  pStream->Write( pOutBuf, pSize );
  pStream->Size = pSize;
  pStream->Seek( 0, soFromBeginning );
  */
  return true;
}

