#ifndef _RPGPC__H
#define _RPGPC__H

#define uint1 unsigned char
#define uint2 unsigned short
#define uint4 unsigned int
#define uint8 unsigned __int64

#define StayDate uint2
#define StayTime uint2

#pragma pack(push, 1)

//***************************************************************************
//      (256)
//***************************************************************************
struct RpgPackHeader
{
  //   --------------------------------------------------
  char      Sign[4];                 //  (PCK)
  uint4     CheckSum;                //   
  uint1     Ver;                     //    
  StayDate  CrtDat;                  //   
  uint4     Len0;                    //    
  uint4     Len1;                    //    
  StayTime  CrtTim;                  //   
  uint4     Size;                    // -    
  uint1     Stt;                     //   
  uint1     DstType;                 //  
  uint1     Kmpl;                    //  
  uint1     NewStt;                  //    (   )
  uint1     PckType;                 //  

  //   ---------------------------------------------
  StayDate  DatPer;                  //  
  char      Code[15];                //  
  char      Name[51];                //  
  uint4     ChkSumFrm;               //      
  char      CrtFIO[51];              //   
  uint1     TypPer;                  //  
  uint1     Prt;                     //  

  //  ---------------------------------------------------------------
  char      FL[51];                  //   
  char      KVK[11];                 // 
  char      XX1[39];                 // <<<  1 >>>
};


// 

#define RPGQWR_SIGN_SIZE 16

#define RPGQWR_SIGN      "ZVIT_QUERY_SIGN"
#define RPGQWRNEW_SIGN   "ZVIT_CRTFC_SIGN"
#define RPGPFU_SIGN      "ZVIT_PFURP_SIGN"

#define CRYPT_SIGN_SIZE  16
#define CRYPT_SIGN      "ZVIT_CRYPT_SIGN"

//   (   )
typedef struct
{
  char  Sign[ RPGQWR_SIGN_SIZE ];  // 
  uint4 Crc;                       // CRC 
  uint1 Direct;               //  
  //      
  union
  {
    //  - 
    struct
    {
      char  From[11];
      uint4 To;
    } PN;
    //  - 
    struct
    {
      uint4 From;
      uint4 To;
    } NN;
    //  - 
    struct
    {
      uint4 From;
      char  To[11];
    } NP;
  } Adr;

  uint1 Type;                 //  
  uint2 CryptKeyNum;          //   (0 - )

} TRpgQwrHeader;

//   
enum
{
  // 
  RPGQWR_DIR          = 0x03, //    
  RPGQWR_DIR_TO_NAL   = 0x01, //  
  RPGQWR_DIR_TO_PLT   = 0x00, //  
  RPGQWR_DIR_FROM_NAL = 0x02, //  
  RPGQWR_DIR_FROM_PLT = 0x00, //  

  RPGQWR_DIR_PLT2NAL  =       //  -> H
    (RPGQWR_DIR_FROM_PLT|RPGQWR_DIR_TO_NAL),
  RPGQWR_DIR_NAL2PLT  =       // H  -> 
    (RPGQWR_DIR_FROM_NAL|RPGQWR_DIR_TO_PLT),
  RPGQWR_DIR_NAL2NAL  =       //   -> 
    (RPGQWR_DIR_FROM_NAL|RPGQWR_DIR_TO_NAL),

  //    -   
  RPGQWR_TYPE_QUERY  = 0x00,  // 
  RPGQWR_TYPE_REPLY  = 0x04,  // 

  //  
  RPGQWR_QWR_PLT2NAL = (RPGQWR_DIR_PLT2NAL|RPGQWR_TYPE_QUERY),
  RPGQWR_REP_PLT2NAL = (RPGQWR_DIR_PLT2NAL|RPGQWR_TYPE_REPLY),
  RPGQWR_QWR_NAL2PLT = (RPGQWR_DIR_NAL2PLT|RPGQWR_TYPE_QUERY),
  RPGQWR_REP_NAL2PLT = (RPGQWR_DIR_NAL2PLT|RPGQWR_TYPE_REPLY),
  RPGQWR_QWR_NAL2NAL = (RPGQWR_DIR_NAL2NAL|RPGQWR_TYPE_QUERY),
  RPGQWR_REP_NAL2NAL = (RPGQWR_DIR_NAL2NAL|RPGQWR_TYPE_REPLY),
};

typedef struct {
  uint4 FrmRcd;
  uint1 EMailable:1;
  uint1 Ei;
  uint4 PerFlg;
  char FrmCd[9];
} FrmEMailNst;

typedef struct
{
  char EDRPOU[11];
  StayDate PckDat;
  StayTime PckTim;
  StayDate RplDat;
  StayTime RplTim;
  StayDate DatPer;
  uint1 TypPer;
  uint4 PckSize;

  uint4 SignSBXSize;
  uint4 CryptSBXSize;
  uint4 RandSBXSize;
  uint4 PqaGFDSize;
  uint4 RplSize;

}PckArchHdr;

typedef struct
{
  char INN[11];
  char FIO[61];
  unsigned char Sex;
  unsigned int CdDoc;
  char NmDoc[51];
  char SerDoc[11];
  char NumDoc[11];
  char VydDoc[51];
  unsigned short DatDoc;
  char NumTel[26];
  char NumDpa[5];
  char Adress[101];
  unsigned short DatBir;
  unsigned int CdNal;
  unsigned char Flg;
} RpgFlcRec;

typedef struct
{
  uint1     Stt;        //  
  uint1     TypPer;     //  
  StayDate  DatPer;     //  
  uint1     Prt;        //  
  StayDate  CrtDat;     //   
  StayDate  SndDat;     //   
  StayDate  CnfDat;     //   
  char      EMail[101]; //  
  char      Where[101]; // . 
  char      PckName[13];//    
  uint1     Prepared;   // 
} RpgPackRec;

#pragma pack(pop)

#endif
